@@ -116,8 +116,14 @@ class ThreadABC(metaclass=abc.ABCMeta):
             reply_to_id: Optional message to reply to
 
         Example:
+            Send a message with a mention to a thread.
+
             >>> mention = fbchat.Mention(thread_id="1234", offset=5, length=2)
-            >>> thread.send_text("A message", mentions=[mention])
+            >>> message_id = thread.send_text("A message", mentions=[mention])
+
+            Reply to the message.
+
+            >>> thread.send_text("A reply", reply_to_id=message_id)
 
         Returns:
             The sent message
@@ -199,7 +205,7 @@ class ThreadABC(metaclass=abc.ABCMeta):
 
             >>> thread.send_location(51.5287718, -0.2416815)
         """
-        await self._send_location(True, latitude=latitude, longitude=longitude)
+        return await self._send_location(True, latitude=latitude, longitude=longitude)
 
     async def send_pinned_location(self, latitude: float, longitude: float):
         """Send a given location to a thread as a pinned location.
@@ -213,7 +219,7 @@ class ThreadABC(metaclass=abc.ABCMeta):
 
             >>> thread.send_pinned_location(39.9390731, 116.117273)
         """
-        await self._send_location(False, latitude=latitude, longitude=longitude)
+        return await self._send_location(False, latitude=latitude, longitude=longitude)
 
     async def send_files(self, files: Iterable[Tuple[str, str]]):
         """Send files from file IDs to a thread.
